USE AdventureWorks2014;
GO

----------------------------------------------

DBCC FREEPROCCACHE
GO

--  Adhoc queries with no autoparameterization
SELECT * FROM [Person].[Address] WHERE [city] = 'London' ;
GO
SELECT * FROM [Person].[Address] WHERE [city] = 'Berlin' ;
GO
SELECT * FROM [Person].[Address] WHERE [city] = 'Orange1' ;
GO


DBCC FREEPROCCACHE
GO

--  Adhoc queries with autoparameterization
SELECT * FROM [Person].[Address] WHERE [Addressid] = 638 ;
GO
SELECT * FROM [Person].[Address] WHERE [Addressid] = 17057 ;
GO
SELECT * FROM [Person].[Address] WHERE [Addressid] = 22330 ;
GO


--  What about dynamic sql
DBCC FREEPROCCACHE
GO

DECLARE @SQL NVARCHAR(200) ;

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [Addressid] = 222' ;
EXEC(@SQL) ;
GO
DECLARE @SQL NVARCHAR(200) ;

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [Addressid] = 333' ;
EXEC(@SQL) ;
GO
DECLARE @SQL NVARCHAR(200) ;

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [Addressid] = 444' ;
EXEC(@SQL) ;
GO

---
DBCC FREEPROCCACHE
GO

DECLARE @SQL NVARCHAR(200) ;

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [city] = ''Orange''' ;
EXEC(@SQL) ;
GO
DECLARE @SQL NVARCHAR(200) ;

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [city] = ''London''' ;
EXEC(@SQL) ;
GO

-------
DBCC FREEPROCCACHE
GO

---  Does sp_executesql make a difference?
DECLARE @City VARCHAR(30) ;
DECLARE @SQL NVARCHAR(500);
DECLARE @ParmDefinition NVARCHAR(500);

SET @SQL = N'SELECT * FROM [Person].[Address] WHERE [city] = @City' ;
SET @ParmDefinition = N'@City VARCHAR(30)';
SET @City = 'London' ;

EXECUTE sp_executesql @SQL, @ParmDefinition, @City = @City;

SET @City = 'Orange' ;
EXECUTE sp_executesql @SQL, @ParmDefinition, @City = @City;
GO


SELECT t.[text], p.size_in_bytes, p.usecounts
FROM sys.dm_exec_cached_plans AS p
CROSS APPLY sys.dm_exec_sql_text(p.plan_handle) AS t
WHERE t.[text] LIKE '%Person%'+'%Address%' AND objtype = 'Adhoc';


/*
    The difference in plan reuse behavior between an Adhoc query
    or plain dynamic sql and using the parameters with sp_executesql
    is comparable to how submitting adhoc sql statements vs. parameterized
    ones from your client app or middle tier. 
    
	Do NOT rely on autoparameterization!
    
	Always use parameters in the connection object or use sp_executesql
    with proper parameters or simply use stored procedures.
*/


