USE AdventureWorks2014;
SET NOCOUNT ON;
GO

-- The column SalesPersonID is the leading key in index IX_SalesOrderHeader_SalesPersonID.
-- Which of the below queries, all of which filter on SalesPersonID, are SARGable,
-- meaning that they will use that index to seek out matching rows?

-- SARGable?  Yes
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID = 283;

-- SARGable?  Depends on SQL version (2008+) and available statistics
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID <> 283;

SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID > 283 OR SalesPersonID < 283;

-- SARGable?  Yes
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID >= 283;

-- SARGable?  No
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID NOT IN 
		(SELECT BusinessEntityID 
		FROM Person.Person 
		WHERE LastName = 'Campbell' 
		AND FirstName = 'David');

-- SARGable?  Yes
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID IN 
		(SELECT BusinessEntityID 
		FROM Person.Person 
		WHERE LastName = 'Campbell' AND FirstName = 'David')

-- SARGable?  No
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE ISNULL(SalesPersonID, 0) = 283

-- SARGable?  Yes
SELECT SalesOrderID 
FROM Sales.SalesOrderHeader 
WHERE SalesPersonID IS NULL

-- Make it complex
SELECT SOH.CustomerID,
	SOH.OrderDate,
	SOH.STATUS,
	SOH.SalesOrderID
	FROM Sales.SalesOrderHeader AS SOH
	INNER JOIN 
		(SELECT CustomerID, MAX (OrderDate) as ODate
		FROM Sales.SalesOrderHeader
		GROUP BY CustomerID) AS MaxOrdList
	ON MaxOrdList.ODate = SOH.OrderDate
	AND MaxOrdList.CustomerID = SOH.CustomerID
WHERE SOH.CustomerID IN (11276, 11201);

-- Usually causes some CXPacket waits
SELECT TOP(1000) *, 
    ROW_NUMBER() OVER 
    ( 
        PARTITION BY OrderQty 
        ORDER BY ProductId 
    ) AS r 
FROM Sales.SalesOrderDetail 
ORDER BY SalesOrderDetailId DESC;